import { AnyIterable } from 'streaming-iterables';
import { Multiaddr } from '@multiformats/multiaddr';
import { PeerId } from '@libp2p/interface-peer-id';
import { Libp2p } from 'libp2p';

type IterableFunc<I, O> = (input: AnyIterable<I>) => AnyIterable<O> | Promise<AnyIterable<O>>;
type Send<T> = (value: T) => Promise<void>;
type Func<I, O> = (input: I, send: Send<O>, done: () => Promise<void>) => Promise<void> | void;
type Service<I, O> = () => Promise<Func<I, O>> | Func<I, O>;
interface Codec<T> {
    encoder: (data: T) => Uint8Array | Promise<Uint8Array>;
    decoder: (data: Uint8Array) => T | Promise<T>;
}
interface InitOptions<T> {
    codec?: Codec<T>;
}
interface TransportChannel<I, O> extends AsyncIterableIterator<I> {
    send: Send<O>;
    done: () => Promise<void>;
}

declare const client: <T = any>(node: Libp2p, peer: string | Multiaddr | PeerId, options?: InitOptions<T>) => Promise<(<I extends T, O extends T>(name: string) => Promise<((value: I) => Promise<O[]>) & TransportChannel<O, I>>)>;

declare const server: <T>(node: Libp2p, options?: InitOptions<T>) => {
    handle: <I extends T, O extends T>(name: string, func: Func<I, O>) => Promise<void>;
    serve: <I_1 extends T, O_1 extends T>(name: string, func: Service<I_1, O_1>) => Promise<void>;
};

export { Codec, Func, InitOptions, IterableFunc, Send, Service, TransportChannel, client, server };
